import ast
import os
import tempfile

import pytest

from pytojsonschema.common import init_typing_namespace, init_schema_map
from pytojsonschema.types import (
    ANY_SCHEMA,
    process_alias,
    process_import,
    process_import_from,
    process_assign,
    process_class_def,
)


@pytest.mark.parametrize(
    "alias_object, expected",
    [
        [ast.parse("import typing as foo").body[0].names[0], "foo"],
        [ast.parse("import typing").body[0].names[0], "typing"],
    ],
    ids=["alias", "no_alias"],
)
def test_process_alias(alias_object, expected):
    assert process_alias(alias_object) == expected


@pytest.mark.parametrize(
    "ast_import, expected",
    [
        [
            ast.parse("import typing as foo").body[0],
            (
                {
                    "Union": {"foo.Union"},
                    "List": {"foo.List"},
                    "Dict": {"foo.Dict"},
                    "Optional": {"foo.Optional"},
                    "Any": {"foo.Any"},
                    "TypedDict": {"foo.TypedDict"},
                    "Enum": set(),
                },
                dict(
                    init_schema_map(),
                    **{
                        "foo.Any": {
                            "anyOf": [
                                {"type": "object"},
                                {"type": "array"},
                                {"type": "null"},
                                {"type": "string"},
                                {"type": "boolean"},
                                {"type": "integer"},
                                {"type": "number"},
                            ]
                        }
                    },
                ),
            ),
        ],
        [
            ast.parse("import enum").body[0],
            (
                {
                    "Union": set(),
                    "List": set(),
                    "Dict": set(),
                    "Optional": set(),
                    "Any": set(),
                    "TypedDict": set(),
                    "Enum": {"enum.Enum"},
                },
                init_schema_map(),
            ),
        ],
        [
            ast.parse("import os").body[0],
            (
                {
                    "Union": set(),
                    "List": set(),
                    "Dict": set(),
                    "Optional": set(),
                    "Any": set(),
                    "TypedDict": set(),
                    "Enum": set(),
                },
                init_schema_map(),
            ),
        ],
    ],
    ids=["typing", "enum", "no_typing_nor_enum"],
)
def test_process_import(ast_import, expected):
    type_namespace, schema_map = init_typing_namespace(), init_schema_map()
    process_import(ast_import, type_namespace, schema_map)
    assert expected[0] == type_namespace
    assert expected[1] == schema_map


@pytest.mark.parametrize(
    "ast_assign, type_namespace, schema_map, expected",
    [
        [
            ast.parse("a = typing.Optional[str]").body[0],
            dict(init_typing_namespace(), **{"Optional": {"typing.Optional"}}),
            init_schema_map(),
            dict(init_schema_map(), **{"a": {"anyOf": [{"type": "string"}, {"type": "null"}]}}),
        ],
        [ast.parse("a = b[34]").body[0], init_typing_namespace(), init_schema_map(), init_schema_map()],
    ],
    ids=["processed", "not_processed"],
)
def test_process_assign(ast_assign, type_namespace, schema_map, expected):
    process_assign(ast_assign, type_namespace, schema_map)
    assert expected == schema_map


@pytest.mark.parametrize(
    "ast_class_def, type_namespace, schema_map, expected",
    [
        [ast.parse("class Foo: pass").body[0], init_typing_namespace(), init_schema_map(), init_schema_map()],
        [ast.parse("class Foo(bar): pass").body[0], init_typing_namespace(), init_schema_map(), init_schema_map()],
        [
            ast.parse(
                """class Car(typing.TypedDict):
    '''Some docstring'''
    model: str = "Ford"
    plate: str"""
            ).body[0],
            dict(init_typing_namespace(), **{"TypedDict": {"typing.TypedDict"}}),
            init_schema_map(),
            dict(
                init_schema_map(),
                **{
                    "Car": {
                        "additionalProperties": False,
                        "properties": {"model": {"type": "string"}, "plate": {"type": "string"}},
                        "required": ["plate"],
                        "type": "object",
                    }
                },
            ),
        ],
        [
            ast.parse(
                """class Color(enum.Enum):
    '''Some docstring'''
    red = 'red'
    blue = 'blue'
    green = 3
    pink = 3.14
    purple = True
    black = None"""
            ).body[0],
            dict(init_typing_namespace(), **{"Enum": {"enum.Enum"}}),
            init_schema_map(),
            dict(init_schema_map(), **{"Color": {"enum": ["red", "blue", 3, 3.14, True, None]}},),
        ],
        [
            ast.parse(
                """class Number(enum.Enum):
    '''Some docstring'''
    three = 3
    four = eval"""
            ).body[0],
            dict(init_typing_namespace(), **{"Enum": {"enum.Enum"}}),
            init_schema_map(),
            init_schema_map(),
        ],
    ],
    ids=["not_typed_dict_nor_enum", "not_typed_dict_nor_enum_with_base_class", "typed_dict", "enum", "enum_bad_types"],
)
def test_process_class_def(ast_class_def, type_namespace, schema_map, expected):
    process_class_def(ast_class_def, type_namespace, schema_map)
    assert expected == schema_map


@pytest.mark.parametrize(
    "ast_import_from, base_path, expected",
    [
        [
            ast.parse("from typing import Any").body[0],
            ".",
            (dict(init_typing_namespace(), **{"Any": {"Any"}}), dict(init_schema_map(), **{"Any": ANY_SCHEMA})),
        ],
        [
            ast.parse("from enum import Enum").body[0],
            ".",
            (dict(init_typing_namespace(), **{"Enum": {"Enum"}}), init_schema_map()),
        ],
        [
            ast.parse("from typing import Union").body[0],
            ".",
            (dict(init_typing_namespace(), **{"Union": {"Union"}}), init_schema_map()),
        ],
        [ast.parse("from enum import foo").body[0], ".", (init_typing_namespace(), init_schema_map())],
        [ast.parse("from typing import foo").body[0], ".", (init_typing_namespace(), init_schema_map())],
        [ast.parse("from os import path").body[0], ".", (init_typing_namespace(), init_schema_map())],
    ],
    ids=["typing_any", "enum_enum", "typing_union", "else_typing", "else_enum", "else_no_typing_nor_enum"],
)
def test_process_import_from_external(ast_import_from, base_path, expected):
    type_namespace, schema_map = init_typing_namespace(), init_schema_map()
    process_import_from(ast_import_from, base_path, type_namespace, schema_map)
    assert expected[0] == type_namespace
    assert expected[1] == schema_map


def test_process_import_from_local():
    with tempfile.TemporaryDirectory() as package:
        # Package creation
        subpackage = os.path.join(package, "subpackage")
        os.mkdir(subpackage)
        package_init = os.path.join(package, "__init__.py")
        package_foo = os.path.join(package, "foo.py")
        subpackage_init = os.path.join(subpackage, "__init__.py")
        subpackage_bar = os.path.join(subpackage, "bar.py")
        subpackage_baz = os.path.join(subpackage, "baz.py")
        with open(package_init, "w") as f:
            f.write("import typing\n\nfrom .foo import B\n\n\nclass A(typing.TypedDict):\n    param: B\n")
        with open(package_foo, "w") as f:
            f.write("import typing\n\n\nclass B(typing.TypedDict):\n    param: int\n")
        with open(subpackage_init, "w") as f:
            f.write("import typing\n\n\nC = typing.Union[bool, float]\n\n\neval(3)")
        with open(subpackage_bar, "w") as f:
            f.write(
                "from .. import A\nfrom ..foo import B\nfrom . import C\nfrom .baz import D\nfrom .baz import bad\n"
            )
        with open(subpackage_baz, "w") as f:
            f.write("import typing\n\n\nD = typing.Dict[str, int]\n")
        # Tests
        type_namespace = init_typing_namespace()
        b_schema = {
            "type": "object",
            "properties": {"param": {"type": "integer"}},
            "required": ["param"],
            "additionalProperties": False,
        }
        a_schema = {
            "type": "object",
            "properties": {"param": b_schema},
            "required": ["param"],
            "additionalProperties": False,
        }
        with open(package_init) as f:
            schema_map = init_schema_map()
            process_import_from(ast.parse(f.read()).body[1], package, type_namespace, schema_map)
            assert schema_map == {
                "bool": {"type": "boolean"},
                "int": {"type": "integer"},
                "float": {"type": "number"},
                "str": {"type": "string"},
                "B": b_schema,
            }
        with open(subpackage_bar) as f:
            file_content = f.read()
            schema_map = init_schema_map()
            process_import_from(ast.parse(file_content).body[0], subpackage, type_namespace, schema_map)
            assert schema_map == {
                "bool": {"type": "boolean"},
                "int": {"type": "integer"},
                "float": {"type": "number"},
                "str": {"type": "string"},
                "A": a_schema,
            }
            schema_map = init_schema_map()
            process_import_from(ast.parse(file_content).body[1], subpackage, type_namespace, schema_map)
            assert schema_map == {
                "bool": {"type": "boolean"},
                "int": {"type": "integer"},
                "float": {"type": "number"},
                "str": {"type": "string"},
                "B": b_schema,
            }
            schema_map = init_schema_map()
            process_import_from(ast.parse(file_content).body[2], subpackage, type_namespace, schema_map)
            assert schema_map == {
                "bool": {"type": "boolean"},
                "int": {"type": "integer"},
                "float": {"type": "number"},
                "str": {"type": "string"},
                "C": {"anyOf": [{"type": "boolean"}, {"type": "number"}]},
            }
            schema_map = init_schema_map()
            process_import_from(ast.parse(file_content).body[3], subpackage, type_namespace, schema_map)
            assert schema_map == {
                "bool": {"type": "boolean"},
                "int": {"type": "integer"},
                "float": {"type": "number"},
                "str": {"type": "string"},
                "D": {"additionalProperties": {"type": "integer"}, "type": "object"},
            }
            schema_map = init_schema_map()
            process_import_from(ast.parse(file_content).body[4], subpackage, type_namespace, schema_map)
            assert schema_map == {
                "bool": {"type": "boolean"},
                "int": {"type": "integer"},
                "float": {"type": "number"},
                "str": {"type": "string"},
            }



import ast
import typing

from .common import (
    TypeNamespace,
    SchemaMap,
    Schema,
    get_ast_name_or_attribute_string,
    VALID_TYPING_AST_SUBSCRIPT_TYPES,
    InvalidTypeAnnotation,
)


def get_json_schema_from_ast_element(
    ast_element: typing.Union[ast.Name, ast.Constant, ast.Attribute, ast.Subscript],
    type_namespace: TypeNamespace,
    schema_map: SchemaMap,
) -> Schema:
    """
    Return the json schema from an type annotation ast object.

    :param ast_element: An ast name, constant, attribute or subscript element
    :param type_namespace: The current typing namespace to be read
    :param schema_map: The current schema map to be read
    :return: A dictionary with the json schema
    """

    def _get_or_raise(element_string: str) -> dict:
        if element_string not in schema_map:
            raise InvalidTypeAnnotation(
                f"Type '{element_string}' is invalid. Base types and the ones you have imported are "
                f"{', '.join(schema_map.keys())}. Did you miss an import?"
            )
        return schema_map[element_string]

    if isinstance(ast_element, ast.Constant) and ast_element.kind is None:
        return {"type": "null"}
    elif isinstance(ast_element, (ast.Name, ast.Attribute)):
        return _get_or_raise(get_ast_name_or_attribute_string(ast_element))
    elif isinstance(ast_element, ast.Subscript):  # typing.List, typing.Dict, typing.Union and typing.Optional
        subscript_string = get_ast_name_or_attribute_string(ast_element.value)
        subscript_type = None
        for key in VALID_TYPING_AST_SUBSCRIPT_TYPES:
            if subscript_string in type_namespace.get(key, {}):
                subscript_type = key
                break
        if subscript_type is None:
            imported_types = []
            for key in VALID_TYPING_AST_SUBSCRIPT_TYPES:
                for element in type_namespace.get(key, {}):
                    imported_types.append(element)
            if imported_types:
                error_msg = (
                    f"Type '{subscript_string}' is invalid. You have imported {', '.join(sorted(imported_types))}, and "
                    f"we allow {', '.join(sorted(list(VALID_TYPING_AST_SUBSCRIPT_TYPES)))}. Did you miss an import?"
                )
            else:
                error_msg = (
                    f"Type '{subscript_string}' is invalid, but no valid types were found. Did you forget importing "
                    f"typing?"
                )
            raise InvalidTypeAnnotation(error_msg)
        if isinstance(ast_element.slice.value, (ast.Constant, ast.Name, ast.Attribute, ast.Subscript)):
            inner_schema = get_json_schema_from_ast_element(ast_element.slice.value, type_namespace, schema_map,)
            if subscript_type == "List":
                return {"type": "array", "items": inner_schema}
            elif subscript_type == "Optional":
                return {"anyOf": [inner_schema, {"type": "null"}]}
            else:
                raise InvalidTypeAnnotation(f"{subscript_type} cannot have a single element")
        else:  # ast.Tuple
            if subscript_type == "Dict":
                if not (
                    isinstance(ast_element.slice.value.elts[0], ast.Name)
                    and ast_element.slice.value.elts[0].id == "str"
                ):
                    raise InvalidTypeAnnotation("typing.Dict keys must be strings")
                return {
                    "type": "object",
                    "additionalProperties": get_json_schema_from_ast_element(
                        ast_element.slice.value.elts[1], type_namespace, schema_map
                    ),
                }
            else:  # Union
                return {
                    "anyOf": [
                        get_json_schema_from_ast_element(element, type_namespace, schema_map)
                        for element in ast_element.slice.value.elts
                    ]
                }
    else:
        raise InvalidTypeAnnotation(f"Unknown type annotation ast element '{str(type(ast_element))}'")



import ast
import functools

import pytest

from pytojsonschema.common import init_schema_map, InvalidTypeAnnotation
from pytojsonschema.jsonschema import get_json_schema_from_ast_element
from pytojsonschema.types import ANY_SCHEMA

from .conftest import assert_expected, TEST_TYPING_NAMESPACE


@pytest.mark.parametrize(
    "ast_element, type_namespace, schema_map, expected",
    [
        [ast.parse("None").body[0].value, TEST_TYPING_NAMESPACE, init_schema_map(), {"type": "null"}],
        [ast.parse("bool").body[0].value, TEST_TYPING_NAMESPACE, init_schema_map(), {"type": "boolean"}],
        [
            ast.parse("complex").body[0].value,
            TEST_TYPING_NAMESPACE,
            init_schema_map(),
            InvalidTypeAnnotation(
                "Type 'complex' is invalid. Base types and the ones you have imported are bool, int, float, str. "
                "Did you miss an import?"
            ),
        ],
        [
            ast.parse("Union[str, int]").body[0].value,
            TEST_TYPING_NAMESPACE,
            init_schema_map(),
            InvalidTypeAnnotation(
                "Type 'Union' is invalid. You have imported typing.Dict, typing.List, typing.Optional, "
                "typing.Union, and we allow Dict, List, Optional, Union. Did you miss an import?"
            ),
        ],
        [
            ast.parse("typing.Union[str, int]").body[0].value,
            {},
            init_schema_map(),
            InvalidTypeAnnotation(
                "Type 'typing.Union' is invalid, but no valid types were found. Did you forget importing typing?"
            ),
        ],
        [
            ast.parse("typing.List[str]").body[0].value,
            TEST_TYPING_NAMESPACE,
            init_schema_map(),
            {"type": "array", "items": {"type": "string"}},
        ],
        [
            ast.parse("typing.Optional[int]").body[0].value,
            TEST_TYPING_NAMESPACE,
            init_schema_map(),
            {"anyOf": [{"type": "integer"}, {"type": "null"}]},
        ],
        [
            ast.parse("typing.Optional[typing.Any]").body[0].value,
            TEST_TYPING_NAMESPACE,
            dict(init_schema_map(), **{"typing.Any": ANY_SCHEMA}),
            {"anyOf": [ANY_SCHEMA, {"type": "null"}]},
        ],
        [
            ast.parse("typing.Union[str]").body[0].value,
            TEST_TYPING_NAMESPACE,
            init_schema_map(),
            InvalidTypeAnnotation("Union cannot have a single element"),
        ],
        [
            ast.parse("typing.Dict[int, str]").body[0].value,
            TEST_TYPING_NAMESPACE,
            init_schema_map(),
            InvalidTypeAnnotation("typing.Dict keys must be strings"),
        ],
        [
            ast.parse("typing.Dict[str, int]").body[0].value,
            TEST_TYPING_NAMESPACE,
            init_schema_map(),
            {"type": "object", "additionalProperties": {"type": "integer"}},
        ],
        [
            ast.parse("typing.Union[int, str]").body[0].value,
            TEST_TYPING_NAMESPACE,
            init_schema_map(),
            {"anyOf": [{"type": "integer"}, {"type": "string"}]},
        ],
        [
            ast.parse("eval(35)").body[0].value,
            TEST_TYPING_NAMESPACE,
            init_schema_map(),
            InvalidTypeAnnotation("Unknown type annotation ast element '<class '_ast.Call'>'"),
        ],
    ],
    ids=[
        "null",
        "base",
        "base_not_found",
        "subscript_missing_import",
        "subscript_missing_typing",
        "list",
        "optional",
        "optional_any",
        "bad_union",
        "dict_bad_keys",
        "dict",
        "union",
        "unsupported",
    ],
)
def test_get_json_schema_from_ast_element(ast_element, type_namespace, schema_map, expected):
    assert_expected(
        functools.partial(get_json_schema_from_ast_element, ast_element, type_namespace, schema_map), expected
    )



import ast
import fnmatch
import logging
import os
import typing

from .common import TypeNamespace, SchemaMap, Schema, init_typing_namespace, init_schema_map
from .jsonschema import get_json_schema_from_ast_element, InvalidTypeAnnotation
from .types import process_import, process_import_from, process_assign, process_class_def

JSON_SCHEMA_DRAFT = "http://json-schema.org/draft-07/schema#"
LOGGER = logging.getLogger()


def process_function_def(
    ast_function_def: ast.FunctionDef, type_namespace: TypeNamespace, schema_map: SchemaMap,
) -> Schema:
    """
    Process a function to return its json schema

    :param ast_function_def: An ast function def element
    :param type_namespace: The current typing namespace to be read
    :param schema_map: The current schema map to be read
    :return: The json schema of the function
    """
    LOGGER.info(f"Processing function {ast_function_def.name} ...")
    # Validation of not supported: Python 3.8 positional-only arguments and *args. Reason: We pass args as key-value
    if getattr(ast_function_def.args, "posonlyargs", None):
        raise InvalidTypeAnnotation(f"Function '{ast_function_def.name}' contains positional only arguments")
    if getattr(ast_function_def.args, "vararg", None):
        raise InvalidTypeAnnotation(
            f"Function '{ast_function_def.name}' contains a variable number positional arguments i.e. *args"
        )
    schema = {
        "$schema": JSON_SCHEMA_DRAFT,
        "type": "object",
        "properties": {},
        "required": [],
        "additionalProperties": False,
    }
    # Process **kwargs
    if ast_function_def.args.kwarg is not None:
        if ast_function_def.args.kwarg.annotation is None:
            raise InvalidTypeAnnotation(
                f"Function '{ast_function_def.name}' is missing its **{ast_function_def.args.kwarg.arg} type annotation"
            )
        schema["additionalProperties"] = get_json_schema_from_ast_element(
            ast_function_def.args.kwarg.annotation, type_namespace, schema_map
        )
    # Positional argument defaults is a non-padded list because you cannot have defaults before non-defaulted args
    # Keyword-only arguments, on the other side, can have defaults at random positions, and the default list is padded
    positional_arg_defaults_padding = len(ast_function_def.args.args) - len(ast_function_def.args.defaults)
    padded_positional_arg_defaults = [None] * positional_arg_defaults_padding + ast_function_def.args.defaults
    for argument, default in zip(
        ast_function_def.args.args + ast_function_def.args.kwonlyargs,
        padded_positional_arg_defaults + ast_function_def.args.kw_defaults,
    ):
        if argument.annotation is None:
            raise InvalidTypeAnnotation(
                f"Function '{ast_function_def.name}' is missing type annotation for the parameter '{argument.arg}'"
            )
        schema["properties"][argument.arg] = get_json_schema_from_ast_element(
            argument.annotation, type_namespace, schema_map
        )
        if default is None:
            schema["required"].append(argument.arg)
    return schema


def filter_by_patterns(
    name: str,
    include_patterns: typing.Optional[typing.List[str]] = None,
    exclude_patterns: typing.Optional[typing.List[str]] = None,
):
    """
    Decide if a name should be included, given a set of include/exclude patterns. These are unix-like patterns,
    implemented with Python's fnmatch https://docs.python.org/3/library/fnmatch.html.

    Exclude patterns override include ones.

    :param name: The name to be filtered
    :param include_patterns: A list of wildcard patterns to match the name you want to include
    :param exclude_patterns: A list of wildcard patterns to match the name you want to exclude
    :return: True if it should be included, False otherwise
    """

    def _is_a_pattern(patterns: typing.List[str]):
        for pattern in patterns:
            if fnmatch.fnmatch(name, pattern):
                return True
        return False

    is_included = True
    is_excluded = False
    if include_patterns:
        is_included = _is_a_pattern(include_patterns)
    if exclude_patterns:
        is_excluded = _is_a_pattern(exclude_patterns)
    if is_excluded or not is_included:
        return False
    return True


def process_file(
    file_path: str,
    include_patterns: typing.Optional[typing.List[str]] = None,
    exclude_patterns: typing.Optional[typing.List[str]] = None,
) -> SchemaMap:
    """
    Process a python file and return all json schemas from the top level functions it can find.

    You can use optional include/exclude patterns to filter the functions you want to process.

    :param file_path: The path to the file
    :param include_patterns: A list of wildcard patterns to match the function names you want to include
    :param exclude_patterns: A list of wildcard patterns to match the function names you want to exclude
    :return: A dictionary containing your function names and their json schemas
    """
    with open(file_path) as f:
        ast_body = ast.parse(f.read()).body
    schema_map = init_schema_map()
    type_namespace = init_typing_namespace()
    function_schema_map = {}

    def _process_function(ast_function_def: ast.FunctionDef):
        if not filter_by_patterns(ast_function_def.name, include_patterns, exclude_patterns):
            LOGGER.info(f"Function {ast_function_def.name} skipped")
        else:
            function_schema_map[ast_function_def.name] = process_function_def(
                ast_function_def, type_namespace, schema_map
            )

    for node in ast_body:
        node_type = type(node)
        process_map = {
            ast.Import: lambda: process_import(node, type_namespace, schema_map),
            ast.ImportFrom: lambda: process_import_from(node, os.path.dirname(file_path), type_namespace, schema_map),
            ast.Assign: lambda: process_assign(node, type_namespace, schema_map),
            ast.ClassDef: lambda: process_class_def(node, type_namespace, schema_map),
            ast.FunctionDef: lambda: _process_function(node),
        }
        if node_type in process_map:
            process_map[node_type]()
    return function_schema_map


def package_iterator(
    package_path: str,
    include_patterns: typing.Optional[typing.List[str]] = None,
    exclude_patterns: typing.Optional[typing.List[str]] = None,
) -> typing.Generator[typing.Tuple[str, str], None, None]:
    """
    Recursively process a package source folder and return the import path prefix and the module file path.

    You can use optional include/exclude patterns to filter the functions you want to process. These patterns are also
    applied to the file names that are processed, with the exception of __init__.py, which is always processed.

    :param package_path: The path to the your python package
    :param include_patterns: A list of wildcard patterns to match the function names you want to include
    :param exclude_patterns: A list of wildcard patterns to match the function names you want to exclude
    :yield: A tuple containing the module import path and the module file path
    """
    norm_package_path = os.path.normpath(package_path)
    path_prefix = os.path.split(norm_package_path)[0]
    for root, _, files in os.walk(norm_package_path, topdown=True):
        for file in files:
            if file.endswith(".py"):
                package_chain = root.replace(path_prefix, "", 1)
                if package_chain.startswith("/"):
                    package_chain = package_chain.replace("/", "", 1)
                package_chain = package_chain.replace(os.path.sep, ".")
                if file != "__init__.py":
                    package_chain = f"{package_chain}.{os.path.splitext(file)[0]}"
                    if not filter_by_patterns(file, include_patterns, exclude_patterns):
                        LOGGER.info(f"Module {package_chain} skipped")
                        continue
                LOGGER.info(f"Module {package_chain}")
                yield package_chain, os.path.join(root, file)


def process_package(
    package_path: str,
    include_patterns: typing.Optional[typing.List[str]] = None,
    exclude_patterns: typing.Optional[typing.List[str]] = None,
) -> SchemaMap:
    """
    Recursively process a package source folder and return all json schemas from the top level functions it can find.

    You can use optional include/exclude patterns to filter the functions you want to process. These patterns are also
    applied to the file names that are processed, with the exception of __init__.py, which is always processed.

    :param package_path: The path to the your python package
    :param include_patterns: A list of wildcard patterns to match the function names you want to include
    :param exclude_patterns: A list of wildcard patterns to match the function names you want to exclude
    :return: A dictionary containing your function names and their json schemas
    """
    function_schema_map = {}
    for package_chain, package_file_path in package_iterator(package_path, include_patterns, exclude_patterns):
        function_schema_map.update(
            **{
                f"{package_chain}.{func_name}": func_schema
                for func_name, func_schema in process_file(
                    package_file_path, include_patterns, exclude_patterns
                ).items()
            }
        )
    return function_schema_map



import os
from setuptools import setup

TEST_DEPENDENCIES = [
    "black==19.10b0",
    "flake8==3.7.9",
    "pytest==5.4.1",
    "pytest-cov==2.8.1",
]

with open(os.path.join(os.path.abspath(os.path.dirname(__file__)), "README.md"), encoding="utf-8") as f:
    LONG_DESCRIPTION = f.read()

setup(
    name="pytojsonschema",
    description="A package to convert Python type annotations into JSON schemas",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    version="1.11.0",
    author="Osirium",
    author_email="support@osirium.com",
    maintainer="Carlos Ruiz Lantero",
    maintainer_email="carlos.ruiz.lantero@gmail.com",
    url="https://github.com/Osirium/pytojsonschema",
    packages=["pytojsonschema"],
    classifiers=[
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    tests_require=TEST_DEPENDENCIES,
    extras_require={"test": TEST_DEPENDENCIES},
)
